package com.rookmotion.rook_sdk_health_connect

import android.app.Activity
import android.content.Context
import com.rookmotion.rook.sdk.RookConfigurationManager
import com.rookmotion.rook.sdk.RookEventManager
import com.rookmotion.rook.sdk.RookHealthPermissionsManager
import com.rookmotion.rook.sdk.RookSummaryManager
import com.rookmotion.rook_sdk_health_connect.extension.genericError
import com.rookmotion.rook_sdk_health_connect.extension.getByteArrayArgAt
import com.rookmotion.rook_sdk_health_connect.extension.getIntArgAt
import com.rookmotion.rook_sdk_health_connect.extension.getLongArgAt
import com.rookmotion.rook_sdk_health_connect.extension.getStringArgAt
import com.rookmotion.rook_sdk_health_connect.extension.successFromBoolean
import com.rookmotion.rook_sdk_health_connect.extension.successFromInt
import com.rookmotion.rook_sdk_health_connect.extension.successFromThrowable
import com.rookmotion.rook_sdk_health_connect.extension.toLocalDate
import com.rookmotion.rook_sdk_health_connect.mapper.toLocalLoggingLevel
import com.rookmotion.rook_sdk_health_connect.mapper.toProto
import com.rookmotion.rook_sdk_health_connect.mapper.toRookConfiguration
import com.rookmotion.rook_sdk_health_connect.proto.ProtobufModels.LocalLoggingLevelProto
import com.rookmotion.rook_sdk_health_connect.proto.ProtobufModels.RookConfigurationProto
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.SupervisorJob
import kotlinx.coroutines.launch
import java.time.Instant

/** RookSdkHealthConnectPlugin */
class RookSdkHealthConnectPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {
    private lateinit var channel: MethodChannel
    private lateinit var scope: CoroutineScope

    private var context: Context? = null
    private var activity: Activity? = null

    private lateinit var rookConfigurationManager: RookConfigurationManager
    private lateinit var rookHealthPermissionsManager: RookHealthPermissionsManager
    private lateinit var rookSummaryManager: RookSummaryManager
    private lateinit var rookEventManager: RookEventManager

    override fun onAttachedToEngine(flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        context = flutterPluginBinding.applicationContext

        rookConfigurationManager = RookConfigurationManager(flutterPluginBinding.applicationContext)
        rookHealthPermissionsManager = RookHealthPermissionsManager(rookConfigurationManager)
        rookSummaryManager = RookSummaryManager(rookConfigurationManager)
        rookEventManager = RookEventManager(rookConfigurationManager)

        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "rook_sdk_health_connect")
        scope = CoroutineScope(SupervisorJob() + Dispatchers.IO)
        channel.setMethodCallHandler(this)
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            "setLocalLoggingLevel" -> {
                val number = call.getIntArgAt(0)
                val proto = LocalLoggingLevelProto.forNumber(number)

                rookConfigurationManager.setLocalLoggingLevel(proto.toLocalLoggingLevel())

                result.successFromBoolean(true)
            }

            "setConfiguration" -> {
                val bytes = call.getByteArrayArgAt(0)
                val proto = RookConfigurationProto.parseFrom(bytes)

                rookConfigurationManager.setConfiguration(proto.toRookConfiguration())

                result.successFromBoolean(true)
            }

            "initRook" -> scope.launch {
                rookConfigurationManager.initRook().fold(
                    {
                        result.successFromBoolean(true)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "updateUserID" -> scope.launch {
                val userID = call.getStringArgAt(0)

                rookConfigurationManager.updateUserID(userID).fold(
                    {
                        result.successFromBoolean(true)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "clearUserID" -> {
                rookConfigurationManager.clearUserID()

                result.successFromBoolean(true)
            }

            "syncUserTimeZone" -> scope.launch {
                rookConfigurationManager.syncUserTimeZone().fold(
                    {
                        result.successFromBoolean(true)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "checkAvailability" -> {
                try {
                    val hcAvailabilityStatus = rookHealthPermissionsManager.checkAvailability(
                        context!!,
                    )
                    val proto = hcAvailabilityStatus.toProto()

                    result.successFromInt(proto.number)
                } catch (e: NullPointerException) {
                    result.genericError(e)
                }
            }

            "openHealthConnectSettings" -> scope.launch {
                rookHealthPermissionsManager.openHealthConnectSettings().fold(
                    {
                        result.successFromBoolean(true)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "hasAllPermissions" -> scope.launch {
                rookHealthPermissionsManager.hasAllPermissions().fold(
                    {
                        result.successFromBoolean(it)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "requestAllPermissions" -> scope.launch {
                try {
                    rookHealthPermissionsManager.requestAllPermissions(activity!!).fold(
                        {
                            result.successFromBoolean(true)
                        },
                        {
                            result.successFromThrowable(it)
                        }
                    )
                } catch (e: NullPointerException) {
                    result.genericError(e)
                }
            }

            "hasSleepPermissions" -> scope.launch {
                rookHealthPermissionsManager.hasSleepPermissions().fold(
                    {
                        result.successFromBoolean(it)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "requestSleepPermissions" -> scope.launch {
                try {
                    rookHealthPermissionsManager.requestSleepPermissions(activity!!).fold(
                        {
                            result.successFromBoolean(true)
                        },
                        {
                            result.successFromThrowable(it)
                        }
                    )
                } catch (e: NullPointerException) {
                    result.genericError(e)
                }
            }

            "hasBodyPermissions" -> scope.launch {
                rookHealthPermissionsManager.hasBodyPermissions().fold(
                    {
                        result.successFromBoolean(it)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "requestBodyPermissions" -> scope.launch {
                try {
                    rookHealthPermissionsManager.requestBodyPermissions(activity!!).fold(
                        {
                            result.successFromBoolean(true)
                        },
                        {
                            result.successFromThrowable(it)
                        }
                    )
                } catch (e: NullPointerException) {
                    result.genericError(e)
                }
            }

            "hasPhysicalPermissions" -> scope.launch {
                rookHealthPermissionsManager.hasPhysicalPermissions().fold(
                    {
                        result.successFromBoolean(it)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "requestPhysicalPermissions" -> scope.launch {
                try {
                    rookHealthPermissionsManager.requestPhysicalPermissions(activity!!).fold(
                        {
                            result.successFromBoolean(true)
                        },
                        {
                            result.successFromThrowable(it)
                        }
                    )
                } catch (e: NullPointerException) {
                    result.genericError(e)
                }
            }

            "shouldSyncSleepSummariesFor" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookSummaryManager.shouldSyncSleepSummariesFor(date).fold(
                    {
                        result.successFromBoolean(it)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "syncSleepSummary" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookSummaryManager.syncSleepSummary(date).fold(
                    {
                        result.successFromBoolean(true)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "shouldSyncBodySummariesFor" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookSummaryManager.shouldSyncBodySummariesFor(date).fold(
                    {
                        result.successFromBoolean(it)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "syncBodySummary" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookSummaryManager.syncBodySummary(date).fold(
                    {
                        result.successFromBoolean(true)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "shouldSyncPhysicalSummariesFor" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookSummaryManager.shouldSyncPhysicalSummariesFor(date).fold(
                    {
                        result.successFromBoolean(it)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "syncPhysicalSummary" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookSummaryManager.syncPhysicalSummary(date).fold(
                    {
                        result.successFromBoolean(true)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "syncPendingSummaries" -> scope.launch {
                rookSummaryManager.syncPendingSummaries().fold(
                    {
                        result.successFromBoolean(true)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "syncPhysicalEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncPhysicalEvents(date).fold(
                    {
                        result.successFromBoolean(true)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "syncBloodGlucoseEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncBloodGlucoseEvents(date).fold(
                    {
                        result.successFromBoolean(true)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "syncBloodPressureEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncBloodPressureEvents(date).fold(
                    {
                        result.successFromBoolean(true)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "syncBodyMetricsEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncBodyMetricsEvents(date).fold(
                    {
                        result.successFromBoolean(true)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "syncBodyHeartRateEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncBodyHeartRateEvents(date).fold(
                    {
                        result.successFromBoolean(true)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "syncPhysicalHeartRateEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncPhysicalHeartRateEvents(date).fold(
                    {
                        result.successFromBoolean(true)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "syncHydrationEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncHydrationEvents(date).fold(
                    {
                        result.successFromBoolean(true)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "syncNutritionEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncNutritionEvents(date).fold(
                    {
                        result.successFromBoolean(true)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "syncBodyOxygenationEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncBodyOxygenationEvents(date).fold(
                    {
                        result.successFromBoolean(true)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "syncPhysicalOxygenationEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncPhysicalOxygenationEvents(date).fold(
                    {
                        result.successFromBoolean(true)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "syncTemperatureEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncTemperatureEvents(date).fold(
                    {
                        result.successFromBoolean(true)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            "syncPendingEvents" -> scope.launch {
                rookEventManager.syncPendingEvents().fold(
                    {
                        result.successFromBoolean(true)
                    },
                    {
                        result.successFromThrowable(it)
                    }
                )
            }

            else -> result.notImplemented()
        }
    }

    override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        initializeActivity(binding)
    }

    override fun onDetachedFromActivityForConfigChanges() {
        destroyActivity()
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        initializeActivity(binding)
    }

    override fun onDetachedFromActivity() {
        destroyActivity()
    }

    private fun initializeActivity(binding: ActivityPluginBinding) {
        try {
            activity = binding.activity
        } catch (ignored: Exception) {
            // Ignored
        }
    }

    private fun destroyActivity() {
        try {
            activity = null
        } catch (ignored: Exception) {
            // Ignored
        }
    }
}
